@@ -18,7 +18,8 @@ import os
 import sys
 import re
 from os import path
-from calculate.lib.datavars import Variable,VariableError,ReadonlyVariable
+from calculate.lib.datavars import (Variable,VariableError,ReadonlyVariable,
+                                    ReadonlyTableVariable, FieldValue)
 from calculate.lib.utils.files import (
         typeFile,process,listDirectory,MAGIC_SYMLINK,MAGIC_COMPRESS)
 
@@ -45,30 +46,47 @@ class VariableOsInstallKernelScheduler(Variable):
         self.help = _("toggle the I/O scheduler")
         self.label = _("I/O scheduler")
 
+    def check_scheduler(self, scheduler):
+        return scheduler in self.Select('os_install_kernel_schedule_name',
+             where='os_install_kernel_schedule_set',
+             eq='on')
+
+    def get_default(self):
+        root_dev = self.Select('os_install_disk_parent',
+                               where='os_install_disk_mount',
+                               eq='/',limit=1)
+        if root_dev:
+            dev_ssd = self.Select('os_device_ssd_set',
+                                   where='os_device_dev',
+                                   eq=root_dev,limit=1)
+            if dev_ssd == 'on' and self.check_scheduler("noop"):
+                return "noop"
+        return self.Get('os_install_kernel_schedule_default')
+
     def get(self):
         """Get scheduler for install root device"""
         if self.Get('os_root_type') == 'livecd':
-            root_dev = self.Select('os_install_disk_parent',
-                                   where='os_install_disk_mount',
-                                   eq='/',limit=1)
-            if root_dev:
-                dev_ssd = self.Select('os_device_ssd_set',
-                                       where='os_device_dev',
-                                       eq=root_dev,limit=1)
-                if dev_ssd == 'on':
-                    return "noop"
+            return self.get_default()
         else:
             currentScheduler = getValueFromCmdLine('elevator',0)
             if currentScheduler in map(lambda x:x[0],self.choice()):
                 return currentScheduler
-        return "cfq"
+        return self.Get('os_install_kernel_schedule_default')
     
     def choice(self):
-        schedulers = [("deadline","Deadline"),("cfq","CFQ"),("noop","No-op")]
-        if self.Get('os_install_kernel_bfq_set') == 'on':
-            return schedulers + [("bfq","BFQ")]
-        else:
-            return schedulers
+        schedulers = {"deadline": "Deadline",
+                      "cfq": "CFQ",
+                      "noop": "No-op",
+                      "bfq": "BFQ"}
+        return [(x, schedulers.get(x,x))
+                for x in self.Select('os_install_kernel_schedule_name',
+                                     where='os_install_kernel_schedule_set',
+                                     eq='on')] + [("auto",_("Default"))]
+
+    def set(self, value):
+        if value == "auto":
+            return self.get_default()
+        return value
 
     def uncompatible(self):
         """
@@ -77,6 +95,79 @@ class VariableOsInstallKernelScheduler(Variable):
         if self.Get('os_install_root_type') == 'flash':
             return _("I/O scheduler unavailable for Flash install")
 
+
+class VariableOsInstallKernelScheduleDefault(Variable):
+    """
+    IO планировщик по умолчанию
+    """
+    type = "choice"
+
+    def get(self):
+        image = self.Get('cl_image')
+        if image:
+            with image as distr:
+                try:
+                    distrPath = image.getDirectory()
+                    kernelConfig = path.join(distrPath,"usr/src/linux/.config")
+                    for line in readLinesFile(kernelConfig):
+                        if "CONFIG_DEFAULT_IOSCHED=" in line:
+                            key, op, value = line.partition("=")
+                            return value.strip('"')
+                except DistributiveError as e:
+                    pass
+        return "cfq"
+
+    def choice(self):
+        return self.Select('os_install_kernel_schedule_name',
+                                     where='os_install_kernel_schedule_set',
+                                     eq='on')
+
+
+class VariableOsInstallKernelScheduleData(ReadonlyTableVariable):
+    """
+    Information about kernel schedule
+    """
+    source = ['os_install_kernel_schedule_name',
+              'os_install_kernel_schedule_set']
+
+    def get(self,hr=False):
+        image = self.Get('cl_image')
+        installed = ['cfq']
+        schedulers = {'CONFIG_IOSCHED_BFQ=y':'bfq',
+                     'CONFIG_IOSCHED_NOOP=y':'noop',
+                     'CONFIG_IOSCHED_CFQ=y':'cfq',
+                     'CONFIG_IOSCHED_DEADLINE=y':'deadline'}
+        if image:
+            with image as distr:
+                try:
+                    distrPath = image.getDirectory()
+                    kernelConfig = path.join(distrPath,"usr/src/linux/.config")
+                    installed =  map(schedulers.get,
+                        filter(lambda x:x in schedulers,
+                        readLinesFile(kernelConfig)))
+                except DistributiveError as e:
+                    pass
+        return [[x,"on" if x in installed else "off"]
+                for x in sorted(schedulers.values())]
+
+    setValue = Variable.setValue
+
+class VariableOsInstallKernelScheduleName(FieldValue,ReadonlyVariable):
+    """
+    Schedule name
+    """
+    type = "list"
+    source_variable = "os_install_kernel_schedule_data"
+    column = 0
+
+class VariableOsInstallKernelScheduleSet(FieldValue,ReadonlyVariable):
+    """
+    Kernel has schedule
+    """
+    type = "list-bool"
+    source_variable = "os_install_kernel_schedule_data"
+    column = 1
+
 class VariableOsInstallKernelBfqSet(ReadonlyVariable):
     """
     Available BFQ in kernel
